<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Config;

use Illuminate\Support\Facades\Log;
use App\Models\{Product, ProviderOauth, Site, User};
use App\Jobs\AccountingItemSync as Job;

class AccountingItemList extends Command
{
	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'accounting:item:list'
		.' {siteid : Site ID}'
		.' {provider : Provider Name}'
		.' {user : User Email}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'Synchronise items with accounting system';

	/**
	 * Execute the console command.
	 *
	 * @return int
	 */
	public function handle()
	{
		$site = Site::findOrFail($this->argument('siteid'));
		Config::set('site',$site);

		$so = ProviderOauth::where('name',$this->argument('provider'))->singleOrFail();
		$uo = User::where('email',$this->argument('user'))->singleOrFail();

		if (($x=$so->tokens->where('user_id',$uo->id))->count() !== 1)
			abort(500,sprintf('Unknown Tokens for [%s]',$uo->email));

		$to = $x->pop();

		// Current Products used by services
		$products = Product::select(['products.*'])
			->distinct('products.id')
			->join('services',['services.product_id'=>'products.id'])
			->where('services.active',TRUE)
			->get();

		$api = $so->API($to,TRUE); // @todo Remove TRUE

		$acc = $api->getItems()->pluck('pid','FullyQualifiedName');

		foreach ($products as $po) {
			if (! $po->accounting)
				$this->error(sprintf('Product [%d](%s) doesnt have accounting set',$po->id,$po->name));

			elseif ($acc->has($po->accounting) === FALSE)
				$this->error(sprintf('Product [%d](%s) accounting [%s] doesnt exist?',$po->id,$po->name,$po->accounting));

			else
				$this->info(sprintf('Product [%d](%s) set to accounting [%s]',$po->id,$po->name,$po->accounting));
		}
	}
}